#ifndef STRICT
#define STRICT
#endif

#include <cassert>
#include "GapiDraw.h"
#include "GapiApplication.h"
#include "resource.h"
#include "sprite.h"

// ----------------------------------------


class CMyApplication : public CGapiApplication
{
protected:
	Sprite user_sprite;
	Sprite robot_sprite;
	CGapiSurface *m_background;
	DWORD m_dwKey;
	DWORD m_dwKills;
	HRESULT hr;

public:
    CMyApplication(const GDAPPCONFIG& config) : CGapiApplication(config) {
		m_background = new CGapiSurface(m_pGapiDraw);
		// initialise user sprite
	user_sprite.X_Pos = 100;
	user_sprite.Y_Pos = 140;
	user_sprite.last_direction = 0;
	user_sprite.X_Disp =10;
	user_sprite.Y_Disp =10;
	user_sprite.bitmap = new CGapiSurface(m_pGapiDraw);
	assert(user_sprite.bitmap != 0);

	// initialise autonomous sprite
	robot_sprite.X_Pos = 100;
	robot_sprite.Y_Pos = 200;
	robot_sprite.last_direction = 0;
	robot_sprite.X_Disp =10;
	robot_sprite.Y_Disp =10;
	robot_sprite.bitmap = new CGapiSurface(m_pGapiDraw);
	assert(robot_sprite.bitmap != 0);
	robot_sprite.status = 0;
	robot_sprite.deadtime = 0;


	// initialise key presses
	m_dwKey = 0;
	//initialise number of kills
	m_dwKills=0;

	
	};

	virtual HRESULT InitInstance()
	{
		return GD_OK;
	}
	
	virtual HRESULT ExitInstance()
	{
		return GD_OK;
	}

	virtual HRESULT CreateSysMemSurfaces(CGapiDisplay* pDisplay, HINSTANCE hInstance)
	{
		// Add code to create surfaces here
		// create background from resource
	HRESULT hr = m_background->CreateSurface(GDSURFACE_SYSTEMMEMORY, hInstance, IDR_PNG1, TEXT("PNG"));

	//set bitmap for user sprite
	hr = user_sprite.bitmap->CreateSurface(GDSURFACE_SYSTEMMEMORY, hInstance, IDB_BITMAP1, NULL);
	assert (hr == GD_OK);

	//set bitmap for autonomous sprite
	hr = robot_sprite.bitmap->CreateSurface(GDSURFACE_SYSTEMMEMORY, hInstance, IDB_BITMAP2, NULL);
	assert (hr == GD_OK);
	hr = robot_sprite.explosion_bitmap->CreateSurface(GDSURFACE_SYSTEMMEMORY, hInstance, IDB_BITMAP3, NULL);
	assert (hr == GD_OK);
	hr = robot_sprite.dead_bitmap->CreateSurface(GDSURFACE_SYSTEMMEMORY, hInstance, IDB_BITMAP4, NULL);
	assert (hr == GD_OK);

		return GD_OK;
	}

	
		virtual HRESULT CreateVidMemSurfaces(CGapiDisplay* pDisplay, HINSTANCE hInstance)
	{
		// Add code to create surfaces stored in video memory here
		return GD_OK;
	}

    
	virtual HRESULT ProcessNextFrame(CGapiSurface* pBackbuffer, DWORD dwFlags)
    {
        // draw background
	HRESULT hrr = pBackbuffer->BltFast(0, 0, m_background, NULL, NULL, NULL);
	assert(hrr == GD_OK); 
	// how big is the screen?
	DWORD dwScreenWidth = pBackbuffer->GetWidth();
	DWORD dwScreenHeight = pBackbuffer->GetHeight();

	// calculate next move for the user sprite 
	MoveSpriteByKeyPresses(dwScreenWidth,dwScreenHeight,
		               user_sprite.bitmap->GetWidth(), user_sprite.bitmap->GetHeight(),
					   user_sprite.X_Pos, user_sprite.Y_Pos,
					   user_sprite.X_Disp, user_sprite.Y_Disp);

	// calculate next move for the autonomous sprite
	MoveSpriteRandomly(robot_sprite.last_direction,dwScreenWidth,dwScreenHeight,
		                   robot_sprite.bitmap->GetWidth(), robot_sprite.bitmap->GetHeight(),
						   robot_sprite.X_Pos, robot_sprite.Y_Pos,
						   robot_sprite.X_Disp, robot_sprite.Y_Disp);

	// do collision detection
	
	POINT intersection = {0, 0};
	
	if (robot_sprite.status==0) // do collision detection only if robot sprite is alive
	{
		hrr = m_background->Intersect(user_sprite.X_Pos, user_sprite.Y_Pos, NULL, user_sprite.bitmap,
			robot_sprite.X_Pos, robot_sprite.Y_Pos, NULL, &intersection);

		 
		if ((intersection.x==-1) && (intersection.y==-1))
		{   // we have a collision
			MessageBeep(0xFFFFFFFF); 
			robot_sprite.status = 1;
			m_dwKills++;
		}
	}

	// draw the robot sprite
	if (robot_sprite.status!=0) robot_sprite.deadtime++; // count how many frames sprite has been dead for
	if (robot_sprite.deadtime==6) robot_sprite.status=2; // sprite has finished exploding and is dead
	if (robot_sprite.deadtime==100) 
	{
		robot_sprite.status=0; // sprite comes alive again after 100 frames
		robot_sprite.deadtime=0;
	}

	switch(robot_sprite.status)
	{
		case 0: // draw normal sprite
				hrr = pBackbuffer->BltFast(robot_sprite.X_Pos, robot_sprite.Y_Pos, 
					robot_sprite.bitmap, NULL, GDBLTFAST_KEYSRC, NULL);
				break;
		case 1: // draw exploding sprite
				hrr = pBackbuffer->BltFast(robot_sprite.X_Pos, robot_sprite.Y_Pos, 
					robot_sprite.explosion_bitmap, NULL, GDBLTFAST_KEYSRC, NULL);
				break;		
		case 2: // draw dead sprite
				hrr = pBackbuffer->BltFast(robot_sprite.X_Pos, robot_sprite.Y_Pos, 
					robot_sprite.dead_bitmap, NULL, GDBLTFAST_KEYSRC, NULL);
				break;		
		default: break;// should never happen - but do zippo
	}

	// draw the two sprites
	hrr = pBackbuffer->BltFast(user_sprite.X_Pos, user_sprite.Y_Pos, user_sprite.bitmap, NULL, GDBLTFAST_KEYSRC, NULL);
	assert(hrr == GD_OK);

	// draw text
	TCHAR str[128];
	_stprintf(str, TEXT("Number of Kills: %d"),m_dwKills);
	hrr = pBackbuffer->DrawText(10, 300, str, m_pDisplay->GetSystemFontBorder(), 0, NULL, 0, NULL);
	assert(hrr == GD_OK);
    
        return S_OK;
	};

	HRESULT CMyApplication::MoveSpriteByKeyPresses(DWORD ScreenWidth, DWORD ScreenHeight,
										   DWORD bitmap_width, DWORD bitmap_height, 
										   DWORD &X, DWORD &Y, DWORD X_Disp, DWORD Y_Disp)
	{
		DWORD prevX = X;
		DWORD prevY = Y;

		if (m_dwKey & 1) // up
		{
			Y = Y- Y_Disp;
		}
		if (m_dwKey & 2) // down
		{
			Y = Y+Y_Disp; 	
		}
		if (m_dwKey & 4) // left
		{
			X = X-X_Disp;
		}
		if (m_dwKey & 8) // right
		{
			X=X+X_Disp;
		}
	// check new position is not outside screen bounds
	// if new position would be outside screen bounds
	if ((X<0) || (X>(ScreenWidth-bitmap_width)) || (Y<0) || (Y>(ScreenHeight-bitmap_height)))
		{
			X=prevX; // don't move the sprite
			Y=prevY;
		}

		return S_OK;
	};


HRESULT CMyApplication::MoveSpriteRandomly(DWORD &last_direction, DWORD ScreenWidth, DWORD ScreenHeight,
										   DWORD bitmap_width, DWORD bitmap_height, 
										   DWORD &X, DWORD &Y, DWORD X_Disp, DWORD Y_Disp)
{
	// calculate new position based on current sprite direction 
	DWORD prevX = X;
	DWORD prevY = Y;
	switch(last_direction)
	{
		case 0: // move N
				Y = Y-Y_Disp; break;
		case 1: // move NE
				Y = Y-Y_Disp; 
				X = X+X_Disp; break;		
		case 2: // move E
				X = X+X_Disp; break;
		case 3: //move SE
				X = X+X_Disp; 
				Y = Y+Y_Disp; break;		
		case 4: //move S
				Y = Y+Y_Disp; break;
		case 5: // move SW
				Y = Y+Y_Disp; 
				X = X-X_Disp; break;
		case 6: // move W
				X = X-X_Disp; break;
		case 7: // move NW
				X = X-X_Disp; 
				Y = Y-Y_Disp; break;	
		default: break;// should never happen - but do zippo
	}

	// if new position would be outside screen bounds
	if ((X<0) || (X>(ScreenWidth-bitmap_width)) || (Y<0) || (Y>(ScreenHeight-bitmap_height)))
	{
		X=prevX; // don't move the sprite
		Y=prevY;
		DWORD new_direction = last_direction;
		while (new_direction==last_direction) // make sure new direction is different from old!
		{
			new_direction = (rand()*(DWORD)8)/RAND_MAX;
		}
		last_direction = new_direction; // change sprite direction
		//MessageBeep(0xFFFFFFFF); // beep when sprite hits screen edge
	}

	return S_OK;
};

HRESULT CMyApplication::KeyDown(DWORD dwKey, GDKEYLIST& keylist)
{
	if (dwKey == keylist.vkA)
	{
		// Key '1' (Calendar button on X5) pressed
		Shutdown();
	}
	else if (dwKey == keylist.vkUp)
	{
		m_dwKey |= 1; // up
	}
	else if (dwKey == keylist.vkDown)
	{
		m_dwKey |= 2; // down
	}
	else if (dwKey == keylist.vkLeft)
	{
		m_dwKey |= 4; // left
	}
	else if (dwKey == keylist.vkRight)
	{
		m_dwKey |= 8; // right
	}
	return S_OK;
};

HRESULT CMyApplication::KeyUp(DWORD dwKey, GDKEYLIST& keylist)
{
	if (dwKey == keylist.vkUp)
	{
		m_dwKey &= ~1; // up
	}
	else if (dwKey == keylist.vkDown)
	{
		m_dwKey &= ~2; // down
	}
	else if (dwKey == keylist.vkLeft)
	{
		m_dwKey &= ~4; // left
	}
	else if (dwKey == keylist.vkRight)
	{
		m_dwKey &= ~8; // right
	}
	return S_OK;
};

HRESULT CMyApplication::Shutdown()
{
	delete user_sprite.bitmap ;
	delete robot_sprite.bitmap;
	return GD_OK;
};

	

virtual ~CMyApplication() {};
};

// ----------------------------------------

int APIENTRY WinMain(HINSTANCE hInst, HINSTANCE hPrevInst,
                     LPTSTR pCmdLine, int nCmdShow)
{
    GDAPPCONFIG config;
    ::ZeroMemory(&config, sizeof(GDAPPCONFIG));

    config.hInstance       = hInst;
    config.pAppTitle       = TEXT("Welcome to GapiDraw!");
    config.dwMaxFPS        = 30;
    config.dwWindowIcon    = IDI_ICON1;

    // Create the application
    CMyApplication simple(config);

    // Start main loop
    return simple.Run();
}
